#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "rateupdater.hh"

template <class Vector, class Matrix, class Function, size_t dim>
RateUpdater<Vector, Matrix, Function, dim>::RateUpdater(
    const Matrices<Matrix>& _matrices, const std::vector<Vector>& _u_initial,
    const std::vector<Vector>& _v_initial, const std::vector<Vector>& _a_initial,
    const std::vector<Dune::BitSetVector<dim>>& _dirichletNodes,
    const std::vector<Function>& _dirichletFunctions)
    : matrices(_matrices),
      u(_u_initial),
      v(_v_initial),
      a(_a_initial),
      dirichletNodes(_dirichletNodes),
      dirichletFunctions(_dirichletFunctions) {}

template <class Vector, class Matrix, class Function, size_t dim>
void RateUpdater<Vector, Matrix, Function, dim>::nextTimeStep() {
  u_o = u;
  v_o = v;
  a_o = a;
  postProcessCalled = false;
}

template <class Vector, class Matrix, class Function, size_t dim>
void RateUpdater<Vector, Matrix, Function, dim>::extractDisplacement(std::vector<Vector>& displacements) const {
  if (!postProcessCalled)
    DUNE_THROW(Dune::Exception, "It seems you forgot to call postProcess!");

  displacements = u;
}

template <class Vector, class Matrix, class Function, size_t dim>
void RateUpdater<Vector, Matrix, Function, dim>::extractVelocity(std::vector<Vector>& velocity) const {
  if (!postProcessCalled)
    DUNE_THROW(Dune::Exception, "It seems you forgot to call postProcess!");

  velocity = v;
}

template <class Vector, class Matrix, class Function, size_t dim>
void RateUpdater<Vector, Matrix, Function, dim>::extractOldVelocity(std::vector<Vector>& oldVelocity) const {
  oldVelocity = v_o;
}

template <class Vector, class Matrix, class Function, size_t dim>
void RateUpdater<Vector, Matrix, Function, dim>::extractAcceleration(std::vector<Vector>& acceleration) const {
  if (!postProcessCalled)
    DUNE_THROW(Dune::Exception, "It seems you forgot to call postProcess!");

  acceleration = a;
}

#include "backward_euler.cc"
#include "newmark.cc"
#include "rateupdater_tmpl.cc"
